<?php

require "aps/2/runtime.php";

/**
 * Class cloud presents application and its global parameters
 * @type("http://aps-standard.org/samples/sample-offering/cloud/1.0")
 * @implements("http://aps-standard.org/types/core/application/1.0","http://odin.com/init-wizard/config/1.0", "http://www.odin.com/gdpr-support/1.0")
 */
class cloud extends APS\ResourceBase
{
    # Link to collection of offers. Pay attention to [] brackets at the end of the @link line.
    /**
     * @link("http://aps-standard.org/samples/sample-offering/offer/1.0[]")
     */
    public $offers;

    # Link to collection of contexts. Pay attention to [] brackets at the end of the @link line.
    /**
     * @link("http://aps-standard.org/samples/sample-offering/context/1.0[]")
     */
    public $contexts;


# Global connection settings will be configured by the provider
# Must be forwarded to the appendpoint to set the connection with the external system
    /**
     * @type(string)
     * @title("URL to service")
     * @description("Cloud management server IP or domain name. This is a demo app, all properties will be ignored.")
     */
    public $apphost;

    /**
     * @type(string)
     * @title("Login")
     * @description("Cloud administrator login")
     */
    public $cloudadmin;

    /**
     * @type(string)
     * @title("Password")
     * @description("Cloud administrator password")
     * @encrypted
     */
    public $cloudpass;

    /**
     * @verb(GET)
     * @path("/getInitWizardConfig")
     * @access(admin, true)
     * @access(owner, true)
     * @access(referrer, true)
     */
    public function getInitWizardConfig()
    {
        $myfile = fopen("./wizard_data.json", "r") or die("Unable to open file!");
        $data = fread($myfile, filesize("./wizard_data.json"));
        fclose($myfile);
        return json_decode($data);
    }

    /**
     * @verb(GET)
     * @path("/testConnection")
     * @param(object,body)
     * @access(admin, true)
     * @access(owner, true)
     * @access(referrer, true)
     */
    public function testConnection($body)
    {
        return "";
    }

    /**
     * @verb(POST)
     * @path("/validateInitWizardData")
     * @param(object,body)
     * @access(admin, true)
     * @access(owner, true)
     * @access(referrer, true)
     */
    public function validateInitWizardData($body)
    {
        return "";
    }

    /**
     * @verb(GET)
     * @path("/canForgetAccount")
     * @param(object,body)
     * @return(string,text/json)
     */
    public function canForgetAccount($body)
    {
        $info = new stdClass();
        $info->status = true;
        return json_encode($info);
    }

    /**
     * @verb(GET)
     * @path("/accountInfo")
     * @param(object,body)
     * @return(string,text/json)
     */
    public function accountInfo($body)
    {
        $identity = new stdClass();
        $identity->uuid = $body;
        $info = array(
            array(
                'groupId' => 'serviceGroupId',
                'key' => 'serviceGroupKey',
                'value' => 'serviceGroupValue'
            ),

            array(
                'groupId' => 'serviceGroupId',
                'key' => 'serviceGroupKeyAnother',
                'value' => 'serviceGroupValueAnother'
            ),

            array(
                'groupId' => 'serviceGroupIdAnother',
                'key' => 'serviceAnotherGroupKey',
                'value' => 'serviceAnotherGroupValue'
            )
        );
        $identity->info = $info;
        return json_encode($identity);
    }

    /**
     * @verb(POST)
     * @path("/forgetAccount")
     * @param(object,body)
     * @return(string,text/json)
     */
    public function forgetAccount($body)
    {
        $info = new stdClass();
        $info->status = true;
        return json_encode($info);
    }

    /**
     * @verb(GET)
     * @path("/canForgetUser")
     * @param(object,body)
     * @return(string,text/json)
     */
    public function canForgetUser($body)
    {
        $info = new stdClass();
        $info->status = true;
        return json_encode($info);
    }

    /**
     * @verb(GET)
     * @path("/userInfo")
     * @param(object,body)
     * @return(string,text/json)
     */
    public function userInfo($body)
    {
        $identity = new stdClass();
        $identity->uuid = $body;
        $info = array(
            array(
                'groupId' => 'serviceGroupId',
                'key' => 'serviceGroupKey',
                'value' => 'serviceGroupValue'
            ),

            array(
                'groupId' => 'serviceGroupId',
                'key' => 'serviceGroupKeyAnother',
                'value' => 'serviceGroupValueAnother'
            ),

            array(
                'groupId' => 'serviceGroupIdAnother',
                'key' => 'serviceAnotherGroupKey',
                'value' => 'serviceAnotherGroupValue'
            )
        );
        $identity->info = $info;
        return json_encode($identity);
    }

    /**
     * @verb(POST)
     * @path("/forgetUser")
     * @param(object,body)
     * @return(string,text/json)
     */
    public function forgetUser($body)
    {
        $info = new stdClass();
        $info->status = true;
        return json_encode($info);
    }

    /**
     * @verb(GET)
     * @path("/groups")
     * @param(object,body)
     * @return(string,text/json)
     */
    public function groups($body)
    {
        $groups = array(
            array(
                'groupId' => 'serviceGroupId',
                'description' => 'service group description',
                array(
                    'keyId' => 'serviceGroupKey',
                    'keyDescription' => 'Key Description'
                ),
                array(
                    'keyId' => 'serviceGroupKeyAnother',
                    'keyDescription' => 'Key Description Another'
                )
            ),

            array(
                'groupId' => 'serviceGroupIdAnother',
                'description' => 'another service group description',
                array(
                    'keyId' => 'serviceAnotherGroupKey',
                    'keyDescription' => 'Key Description from second group'
                )
            )
        );
        return json_encode($groups);
    }
}
