<?php

require "aps/2/runtime.php";

// Definition of type structures

class OS
{
    /**
     * @type("string")
     * @title("OS Name")
     * @description("Operating System Name")
     */
    public $name;

    /**
     * @type("string")
     * @title("OS Version")
     * @description("Operating System version")
     */
    public $version;
}

class Hardware
{
    /**
     * @type("integer")
     * @title("RAM Size")
     * @description("RAM size in GB")
     */
    public $memory;

    /**
     * @type("integer")
     * @title("Disk Space")
     * @description("Disk space in GB")
     */
    public $diskspace;

    /**
     * @type("integer")
     * @title("CPU")
     * @description("Server CPU parameters")
     */
    public $CPU;
}

class Platform
{
    /**
     * @type("string")
     * @title("Architecture")
     * @description("Platform architecture")
     */
    public $arch;

    /**
     * @type("OS")
     * @title("OS Parameters")
     * @description("Parameters of operating system")
     */
    public $OS;
}


// Main class
/**
 * @type("http://aps-standard.org/samples/sample-offering/vps/1.0")
 * @implements("http://aps-standard.org/types/core/user/service/1.0")
 */
class vps extends APS\ResourceBase
{

    // Relation with the management context
    /**
     * @link("http://aps-standard.org/samples/sample-offering/context/1.0")
     * @required
     */
    public $context;

    // Relation with an offer
    /**
     * @link("http://aps-standard.org/samples/sample-offering/offer/1.0")
     * @required
     */
    public $offer;

    // Modify the relation with a service user
    /**
     * @link("http://parallels.com/aps/types/pa/user/1.0")
     * @required
     * @assign.access(owner,true)
     */
    public $user;

    // VPS properties

    /**
     * @type("string")
     * @title("name")
     * @description("Server Name")
     */
    public $name;

    /**
     * @type("string")
     * @title("Description")
     * @description("Server Description")
     */
    public $description;

    /**
     * @type("string")
     * @title("state")
     * @description("Server State")
     */
    public $state;

    // VPS complex properties (structures) - defined as classes above

    /**
     * @type("Hardware")
     * @title("Hardware")
     * @description("Server Hardware")
     */
    public $hardware;

    /**
     * @type("Platform")
     * @title("Platform")
     * @description("OS Platform")
     */
    public $platform;

    /**
     * @type("OS")
     * @title("OS Parameters")
     * @description("Parameters of operating system")
     */
    public $OS;

    /**
     * @type("string")
     * @title("User Name")
     * @description("VPS Owner Name")
     */
    public $userName;

    ## Set initial status to "activating" to get redirected to the activation wizard
    public function provision()
    {
        $this->aps->status = "aps:activating";
    }

    ## Activate the VPS - the Activation Wizard calls this method
    /**
     * @verb(PUT)
     * @path("/activate")
     * @param(object,body)
     * @access(admin, true)
     * @access(owner, true)
     * @access(referrer, true)
     */
    public function activate($body)
    {
        $this->aps->status = "aps:ready";
        $this->name = $body->name;
        $apsc = \APS\Request::getController();
        $apsc->updateResource($this);
        return $this;
    }

    // Custom functions called on pressing start/stop buttons in custom UI:
    /**
     * @verb(GET)
     * @path("/start")
     * @return(string,text/json)
     */
    public function start()
    {
        $this->state = 'Running';
        $apsc = \APS\Request::getController();
        $apsc->updateResource($this);
    }

    /**
     * @verb(GET)
     * @path("/stop")
     * @return(string,text/json)
     */
    public function stop()
    {
        $this->state = 'Stopped';
        $apsc = \APS\Request::getController();
        $apsc->updateResource($this);
    }

}

