define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/when",
    "dojox/mvc/getPlainValue",
    "dojox/mvc/at",
    "dojox/mvc/getStateful",

    "aps/ResourceStore",
    "aps/Memory",
    "aps/_View",

    "dojo/text!./newvps.json",
    "dojo/text!./newoffer.json"
], function (declare,
             lang,
             when,
             getPlainValue,
             at,
             getStateful,
             Store,
             Memory,
             _View,
             newVPS,
             newOffer) {
    return declare(_View, {
        init: function () {
            /* Declare the data source */
            this.offerStore = new Store({
                target: "/aps/2/resources",
                apsType: "http://aps-standard.org/samples/sample-offering/offer/1.0"
            });

            this.offerCache = new Memory({
                data: [],
                idProperty: "aps.id"
            }); // Stores a list of offers

            this.offerCtrl = getStateful({
                data: JSON.parse(newOffer)
            }); // Selected offer

            this.model = getStateful({
                data: JSON.parse(newVPS)
            }); // VPS limits

            var self = this;

            /* Handler on offer change */
            function onChangeOffer(offerID) {
                /* console.log(offerID); For debugging only */
                self.offerCtrl.set("data", self.offerCache.get(offerID));
                self.model.data.offer.aps.set("id", self.offerCtrl.data.aps.id);
                self.model.data.platform.OS.set("name", self.offerCtrl.data.platform.OS.name);
            }

            /* Define and return widgets */
            return ["aps/Container", {
                id: this.genId("wizardNew_contaner"),
                title: "Create VPS"
            }, [
                ["aps/Panel", {id: this.genId("wizardNew_panel_general"),title: "Server"}, [
                    ["aps/FieldSet", {id: this.genId("wizardNew_general")}, [
                        ["aps/TextBox", {
                            id: this.genId("wizardNew_serverName"),
                            gridSize: 'md-3 xs-12',
                            label: _("Server Name"),
                            value: at(this.model.data, "name"),
                            required: true
                        }],
                        ["aps/TextBox", {
                            id: this.genId("wizardNew_description"),
                            gridSize: 'md-3 xs-12',
                            label: _("Description"),
                            value: at(this.model.data, "description")
                        }]
                    ]]
                ]],
                ["aps/Panel", {id: this.genId("wizardNew_panel_offers"), title: _("Offer")}, [
                    ["aps/FieldSet", {id: this.genId("wizardNew_offers")}, [
                        ["aps/Select", {
                            id: this.genId("wizardNew_offerSelect"),
                            gridSize: 'md-3 xs-12',
                            required: true,
                            label: _("Bundle"),
                            store: this.offerCache,
                            onChange: onChangeOffer,
                            labelAttr: "profileName",
                            value: at(this.model.data.offer.aps, "id")
                        }]
                    ]]
                ]],
                ["aps/Panel", {id: this.genId("wizardNew_props_panel"),title: "Configuration"}, [
                    ["aps/FieldSet", {id: this.genId("wizardNew_props")}, [
                        ["aps/Select", {
                            id: this.genId("wizardNew_os"),
                            gridSize: 'md-3 xs-12',
                            required: true,
                            disabled: true,
                            label: _("OS"),
                            value: at(this.model.data.platform.OS, "name")
                        }],
                        ["aps/Spinner", {
                            id: this.genId("wizardNew_cpu"),
                            gridSize: 'md-3 xs-12',
                            label: _("CPU Number"),
                            minimum: 1,
                            maximum: at(this.offerCtrl.data.hardware, "CPU"),
                            value: at(this.model.data.hardware, "CPU"),
                            step: 1
                        }],
                        ["aps/Spinner", {
                            id: this.genId("wizardNew_disk"),
                            gridSize: 'md-3 xs-12',
                            label: _("Disk Space"),
                            minimum: 1,
                            maximum: at(this.offerCtrl.data.hardware, "diskspace"),
                            value: at(this.model.data.hardware, "diskspace"),
                            legend: _("Gb"),
                            step: 1
                        }],
                        ["aps/Spinner", {
                            id: this.genId("wizardNew_ram"),
                            gridSize: 'md-3 xs-12',
                            label: _("RAM"),
                            minimum: 128,
                            maximum: at(this.offerCtrl.data.hardware, "memory"),
                            value: at(this.model.data.hardware, "memory"),
                            step: 128,
                            legend: _("Mb")
                        }]
                    ]]
                ]]
            ]];
        }, // End of Init

        /* Create the handler for the Next navigation button */
        onNext: function () {
            var page = this.byId("apsPageContainer");

            page.get("messageList").removeAll();

            if (!page.validate()) {
                aps.apsc.cancelProcessing();
                return;
            }

            aps.biz.getResourcesToBind().then(
                function (resources) {
                    var users = resources.users;
                    return aps.biz.requestUsageChange({
                        deltas: [{
                            apsType: "http://aps-standard.org/samples/sample-offering/vps/1.0",
                            delta: users.length
                        }],
                        operations: users.map(function (user) {
                            return {
                                provision: lang.mixin(getPlainValue(this.model.data), {
                                    user: user,
                                    userName: user.fullName
                                })
                            };
                        }, this),
                        subscriptionId: aps.context.vars.context.aps.subscription
                    });
                }.bind(this)
            ).then(function () {
                aps.apsc.next();
            });
        }, // End of onNext

        onContext: function () {
            var self = this;
            self.offerStore.query().then(function (offers) {
                if (offers.length === 0) return;
                self.offerCache.setData(offers);
                self.offerCtrl.set("data", offers[0]);
                self.model.data.offer.aps.set("id", self.offerCtrl.data.aps.id);
                self.byId("wizardNew_offerSelect").set("store", self.offerCache);
            }).then(aps.apsc.hideLoading());
        } // End of onContext

    }); // End of Declare
}); // End of Define