define([
	"dojo/_base/declare",
	"dojo/promise/all",
	"dojo/when",
	"dojox/mvc/getPlainValue",
	"dojox/mvc/getStateful",
	"dojox/mvc/at",

	"aps/ResourceStore",
	"aps/Memory",
	"aps/xhr",
	"aps/_View",

	"./displayError.js",

	"dojo/text!./newoffer.json",
	"dojo/text!./newvps.json"
],
function(
	declare,
	all,
	when,
	getPlainValue,
	getStateful,
	at,

	Store,
	Memory,
	xhr,
	_View,

	displayError,

	newOffer,
	newVPS
) {

	return declare(_View, {
		init: function() {
			/* Declare data sources */
			this.vpsStore = new Store({ // Interface with the VPS stored in the APS DB
				apsType: "http://aps-standard.org/samples/sample-offering/vps/1.0",
				target: "/aps/2/resources"
			});

			this.offerCache = new Memory({
				idProperty: "aps.id"
			}); // Cache with a list of offers

			this.offerCtrl = getStateful({
				model: JSON.parse(newOffer)
			}); // Stores current offer

			aps.app.model.set("editVPS", getStateful(JSON.parse(newVPS))); // VPS limits that cannot be higher than OfferCtrl values
			this.vps = getStateful({}); /* Selected VPS */
			this.userSelectionList = new Memory({
				idProperty: "aps.id"
			}); /* A list of all service users */

			/* Create widgets */
			return [
				["aps/Panel", {title: _("Server")}, [
					["aps/FieldSet", {id: this.genId("srvEdit_general")}, [
						["aps/TextBox", {
							id: this.genId("srvEdit_serverName"),
							gridSize: 'md-3 xs-12',
							label: _("Server Name"),
							value: at(aps.app.model.editVPS, "name"),
							required: true
						}],
						["aps/TextBox", {
							id: this.genId("srvEdit_description"),
							gridSize: 'md-3 xs-12',
							label: _("Description"),
							value: at(aps.app.model.editVPS, "description")
						}]
					]]
				]],
				["aps/Panel", {title: _("Owner")}, [
					["aps/FieldSet", {id: this.genId("srvEdit_owner")}, [
						["aps/Select", {
							id: this.genId('srvEdit_userSelect'),
							gridSize: 'md-3 xs-12',
							required: true,
							label: _("User Name"),
							labelAttr: "fullName",
							value: at(aps.app.model.editVPS.user.aps, "id")
						}]
					]]
				]],
				["aps/Panel", {}, [
					["aps/FieldSet", {
						id: this.genId("srvEdit_props")
					}, [
						["aps/Select", {
							id: this.genId("srvEdit_os"),
							gridSize: 'md-3 xs-12',
							label: _("OS"),
							disabled: true,
							required: true,
							value: at(this.offerCtrl.model.platform.OS, "name")
						}],
						["aps/Spinner", {
							id: this.genId("srvEdit_cpu"),
							gridSize: 'md-3 xs-12',
							label: _("CPU Cores"),
							minimum: 1,
							maximum: at(this.offerCtrl.model.hardware.CPU, "number"),
							value: at(aps.app.model.editVPS.hardware, "CPU"),
							step: 1
						}],
						["aps/Spinner", {
							id: this.genId("srvEdit_disk"),
							gridSize: 'md-3 xs-12',
							label: _("Disk Space"),
							minimum: 1,
							maximum: at(this.offerCtrl.model.hardware, "diskspace"),
							value: at(aps.app.model.editVPS.hardware, "diskspace"),
							legend: _("GB"),
							step: 1
						}],
						["aps/Spinner", {
							id: this.genId("srvEdit_ram"),
							gridSize: 'md-3 xs-12',
							label: _("RAM"),
							minimum: 128,
							maximum: at(this.offerCtrl.model.hardware, "memory"),
							value: at(aps.app.model.editVPS.hardware, "memory"),
							step: 128,
							legend: _("MB")
						}]
					]]
				]]
			];
		}, // End of init

		/* Define handlers for the navigation buttons */
		onCancel: function() {
			aps.apsc.gotoView("servers");
		},

		onSubmit: function() {
			var page = this.byId("apsPageContainer");
			page.get("messageList").removeAll();
			if (!page.validate()) {
				aps.apsc.cancelProcessing();
				return;
			}

			aps.app.model.editVPS.userName = this.userSelectionList.get(aps.app.model.editVPS.user.aps.id).fullName;
			var data = getPlainValue(aps.app.model.editVPS),
				reqs = []; /* We may have one or two REST requests */

			/* If the User has to be changed, we need to re-link the VPS to the other User */
			if (data.user.aps.id != this.vps.user.aps.id) {
				reqs.push(xhr("/aps/2/resources/" + aps.context.vars.vps.aps.id + "/user", {
					method: "POST",
					data: JSON.stringify({
						"aps": {
							"id": data.user.aps.id
						}
					})
				}));
			}

			data.platform.OS.name = this.offerCtrl.model.platform.OS.name;

			reqs.push(this.vpsStore.put(data)); /* And now update the VPS properties */
			all(reqs).then(function() { /* Once the requests are completed, navigate to the "servers" view */
					aps.apsc.gotoView("servers");
				},
				displayError
			);
		}, // End of onSubmit

		onContext: function(context) {
			var self = this;

			var offerStore = new Store({ // The list of offers for selection
				apsType: "http://aps-standard.org/samples/sample-offering/offer/1.0",
				target: "/aps/2/resources/"
			});

			var userStore = new Store({ // The list of users for selection
				apsType: "http://aps-standard.org/types/core/user/1.0",
				target: "/aps/2/resources/"
			});

			/* Request the list of offers and list of users. Initialize data sources. */
			all([offerStore.query(), userStore.query(), self.vpsStore.get(context.vars.vps.aps.id)])
				.then(function(offersUsersVpses) {
					var offers = offersUsersVpses[0];
					if (offers.length === 0) {
						return;
					}

					var users = offersUsersVpses[1];
					self.userSelectionList.setData(users);
					self.vps = offersUsersVpses[2];
					aps.app.model.set("editVPS", getStateful(offersUsersVpses[2]));

					self.offerCache.setData(offers);
					self.offerCtrl.set("model", self.offerCache.get(self.vps.offer.aps.id));
					self.byId("srvEdit_userSelect").set("store", self.userSelectionList);

					aps.apsc.hideLoading();
				});
		}, // End of onContext

		onHide: function() {
			/* Reset widget fields to avoid flickering when opening another VPS */
			aps.app.model.set("editVPS", getStateful(JSON.parse(newVPS)));
		}

	}); // End of Declare
}); // End of Define